==> trircd-release-5.1.7-r.tar.gz: trircd-release-5.1.7-r/server/s_misc.c <==
/************************************************************************
 *   IRC - Internet Relay Chat, server/s_misc.c
 *
 *   Copyright (C) 2000-2002 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * $Id: s_misc.c,v 5.6 2003/03/18 14:39:01 tr-ircd Exp $ 
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "h.h"
#include "numeric.h"

static char *months[] = {
    "January", "February", "March", "April",
    "May", "June", "July", "August",
    "September", "October", "November", "December"
};

static char *weekdays[] = {
    "Sunday", "Monday", "Tuesday", "Wednesday",
    "Thursday", "Friday", "Saturday"
};

/*
 * stats stuff
 */
char *date(time_t aClock)
{
static char buf[80], plus;
struct tm *lt, *gm;
struct tm gmbuf;
int minswest;

    if (!aClock)
	time(&aClock);
    gm = gmtime(&aClock);
    memcpy((char *) &gmbuf, (char *) gm, sizeof(gmbuf));
    gm = &gmbuf;
    lt = localtime(&aClock);

    if (lt->tm_yday == gm->tm_yday)
	minswest = (gm->tm_hour - lt->tm_hour) * 60 + (gm->tm_min - lt->tm_min);
    else if (lt->tm_yday > gm->tm_yday)
	minswest = (gm->tm_hour - (lt->tm_hour + 24)) * 60;
    else
	minswest = ((gm->tm_hour + 24) - lt->tm_hour) * 60;

    plus = (minswest > 0) ? '-' : '+';
    if (minswest < 0)
	minswest = -minswest;

    (void) ircsprintf(buf, "%s %s %d %04d -- %02d:%02d %c%02d:%02d",
		      weekdays[lt->tm_wday], months[lt->tm_mon],
		      lt->tm_mday, lt->tm_year + 1900, lt->tm_hour,
		      lt->tm_min, plus, minswest / 60, minswest % 60);

    return buf;
}

/*
 */
char *smalldate(time_t aClock)
{
static char buf[MAX_DATE_STRING];
struct tm *lt, *gm;
struct tm gmbuf;

    if (!aClock)
	time(&aClock);
    gm = gmtime(&aClock);
    memcpy((char *) &gmbuf, (char *) gm, sizeof(gmbuf));
    gm = &gmbuf;
    lt = localtime(&aClock);

    (void) ircsprintf(buf, "%04d/%02d/%02d %02d.%02d",
		      lt->tm_year + 1900, lt->tm_mon + 1, lt->tm_mday, lt->tm_hour, lt->tm_min);

    return buf;
}

/**
 ** myctime()
 **   This is like standard ctime()-function, but it zaps away
 **   the newline from the end of that string. Also, it takes
 **   the time value as parameter, instead of pointer to it.
 **   Note that it is necessary to copy the string to alternate
 **   buffer (who knows how ctime() implements it, maybe it statically
 **   has newline there and never 'refreshes' it -- zapping that
 **   might break things in other places...)
 **
 **/

char *myctime(time_t value)
{
static char buf[28];
char *p;

    (void) strcpy(buf, ctime(&value));
    if ((p = (char *) strchr(buf, '\n')) != NULL)
	*p = '\0';

    return buf;
}

/*
 * * get_client_name *      Return the name of the client for various
 * tracking and *      admin purposes. The main purpose of this
 * function is to *      return the "socket host" name of the client,
 * if that *    differs from the advertised name (other than case). *
 * But, this can be used to any client structure. *
 * 
 *      Returns: *        "name[user@ip#.port]" if 'showip' is true; *
 * "name[sockethost]", if name and sockhost are different and *
 * showip is false; else *        "name". *
 * 
 * NOTE 1: *    Watch out the allocation of "nbuf", if either
 * sptr->name * or sptr->sockhost gets changed into pointers instead of *
 * directly allocated within the structure... *
 * 
 * NOTE 2: *    Function return either a pointer to the structure
 * (sptr) or *  to internal buffer (nbuf). *NEVER* use the returned
 * pointer *    to modify what it points!!!
 */
char *get_client_name(aClient *sptr, int showip)
{
    static char nbuf[HOSTLEN * 2 + USERLEN + 5];

    if (MyConnect(sptr)) {
	switch (showip) {
	    case TRUE:
		ircsprintf(nbuf, "%^C [%s]", sptr, sptr->hostip);
		break;
	    case HIDEME:
		ircsprintf(nbuf, "%^C", sptr);
		break;
	    default:
		if (irc_strcmp(sptr->name, sptr->sockhost))
		    ircsprintf(nbuf, "%s[%s]", sptr->name, sptr->sockhost);
		else
		    return sptr->name;
	}
	return nbuf;
    }
    return sptr->name;
}

char *get_client_host(aClient *cptr)
{
    static char nbuf[HOSTLEN * 2 + USERLEN + 5];

    if (!MyConnect(cptr))
	return cptr->name;
    if (!cptr->hostp)
	return get_client_name(cptr, FALSE);
    else
	ircsprintf(nbuf, "%s[%-.*s@%-.*s]",
			  cptr->name, USERLEN,
			  (!(cptr->flags & FLAGS_GOTID)) ? "" : cptr->
			  username, HOSTLEN, cptr->hostp->h_name);
    return nbuf;
}

int add_dccallow(aClient *sptr, aClient *cptr)
{
    dlink_node *ptr;
    int cnt;

    if (!IsPerson(cptr) || !IsPerson(sptr))
        return 0;
    cnt = dlink_list_length(&(sptr->user->dccallow));

    if (dlinkFind(&(sptr->user->dccallow), cptr))
        return 0;
    if (++cnt >= MAXDCCALLOW) {
        send_me_numeric(sptr, ERR_TOOMANYDCC, cptr->name, MAXDCCALLOW);
        return 0;
    }
    ptr = make_dlink_node();
    dlinkAdd(cptr, ptr, &(sptr->user->dccallow));
    ptr = make_dlink_node();
    dlinkAdd(sptr, ptr, &(cptr->user->allowed_by));
    send_me_numeric(sptr, RPL_DCCSTATUS, cptr->name, "added to");
    return 0;
}

int del_dccallow(aClient *sptr, aClient *optr)
{
    dlink_node **inv, *tmp;

    for (inv = &(sptr->user->dccallow.head); (tmp = *inv); inv = &tmp->next) {
        if (tmp->data == optr) {
            dlinkDeleteNode(tmp, &(sptr->user->dccallow));
            break;   
        }
    }
    for (inv = &(optr->user->allowed_by.head); (tmp = *inv); inv = &tmp->next) {
        if (tmp->data == sptr) {
            dlinkDeleteNode(tmp, (&(optr->user->allowed_by)));
            break;
        }
    }
    send_me_numeric(sptr, RPL_DCCSTATUS, optr->name, "removed from");
    return 0;
}

int remove_dcc_references(aClient *sptr)
{
    aClient *acptr;
    dlink_node *lp, *nextlp;
    dlink_node **lpp, *tmp, *next_tmp;
    lp = sptr->user->allowed_by.head;
    while (lp) {
        nextlp = lp->next;
        acptr = lp->data;
        for (lpp = &(acptr->user->dccallow.head); (tmp = *lpp); lpp = &next_tmp) {
	    next_tmp = tmp->next;
            if (tmp->data == sptr) 
                dlinkDeleteNode(tmp, &(acptr->user->dccallow));
        }
        dlinkDeleteNode(lp, &(sptr->user->allowed_by));
	lp = nextlp;
    }
    lp = sptr->user->dccallow.head;
    while (lp) {
        nextlp = lp->next;
        acptr = lp->data;
        for (lpp = &(acptr->user->allowed_by.head); (tmp = *lpp); lpp = &next_tmp) {
	    next_tmp = tmp->next;
            if (tmp->data == sptr) {
                sendto_one(acptr,
                           ":%C %N %s :%s has been removed from your DCC allow list for signing off",
                           &me, RPL_DCCINFO, acptr->name, sptr->name);
                dlinkDeleteNode(tmp, &(acptr->user->allowed_by));
            }
        }
        dlinkDeleteNode(lp, &(sptr->user->dccallow));
	lp = nextlp;
    }
    return 0;
}

char *exploits_2char[] = { "js", "pl", NULL };
char *exploits_3char[] = { "exe", "com", "bat", "dll", "ini", "vbs", "pif", "mrc", "scr",
    "doc", "xls", "lnk", "shs",
    "htm", NULL
};
char *exploits_4char[] = { "html", NULL };

int check_dccsend(aClient *from, aClient *to, char *msg)
{
    /*
     * we already know that msg will consist of "DCC SEND" so we can skip to the end
     */
    char *filename = msg + 8;
    char *ext;
    char **farray;
    int arraysz;
    int len = 0, extlen = 0, i;

    /*
     * people can send themselves stuff all the like..
     * * opers need to be able to send cleaner files
     * * sanity checks..
     */

    if (from == to || !IsPerson(from) || IsAnOper(from) || !MyClient(to))
	return 0;

    while (*filename == ' ')
	filename++;

    if (!(*filename))
	return 0;

    while (*(filename + len) != ' ') {
	if (!(*(filename + len)))
	    break;
	len++;
    }
    for (ext = filename + len;; ext--) {
	if (ext == filename)
	    return 0;

	if (*ext == '.') {
	    ext++;
	    extlen--;
	    break;
	}
	extlen++;
    }
    switch (extlen) {
	case 2:
	    farray = exploits_2char;
	    arraysz = 2;
	    break;

	case 3:
	    farray = exploits_3char;
	    arraysz = 3;
	    break;

	case 4:
	    farray = exploits_4char;
	    arraysz = 4;
	    break;

	    /*
	     * no executable file here..
	     */
	default:
	    return 0;
    }

    for (i = 0; farray[i]; i++) {
	if (irc_strncmp(farray[i], ext, arraysz) == 0)
	    break;
    }

    if (farray[i] == NULL)
	return 0;

    if (!allow_dcc(to, from)) {
    char tmpext[8];
    char tmpfn[128];
    dlink_node *tlp, *flp;
    aChannel *chptr = NULL;
    aChannel *achptr = NULL;
    aChannel *bchptr = NULL;

	strlcpy_irc(tmpext, ext, extlen);
	tmpext[extlen] = '\0';

	if (len > 127)
	    len = 127;
	strlcpy_irc(tmpfn, filename, len);
	tmpfn[len] = '\0';

	/*
	 * use notices!
	 * *   server notices are hard to script around.
	 * *   server notices are not ignored by clients.
	 */

	send_me_notice(from,
		       ":The user %s is not accepting DCC sends of filetype *.%s from you."
		       " Your file %s was not sent.", to->name, tmpext, tmpfn);

	send_me_notice(to,
		       ":%s (%s@%s) has attempted to send you a file named %s, which was blocked.",
		       from->name, from->user->username, from->user->host, tmpfn);

	if (!SeenDCCNotice(to)) {
	    SetDCCNotice(to);

	    send_me_notice(to,
			   ":The majority of files sent of this type are malicious virii and trojan horses."
			   " In order to prevent the spread of this problem, we are blocking DCC sends of"
			   " these types of files by default.");
	    send_me_notice(to,
			   ":If you trust %s, and want him/her to send you this file, you may obtain"
			   " more information on using the dccallow system by typing /dccallow help",
			   from->name, to->name);
	}
	for (tlp = to->user->channel.head; tlp && !chptr; tlp = tlp->next) {
	    achptr = tlp->data;
	    for (flp = from->user->channel.head; flp && !chptr; flp = flp->next) {
		bchptr = flp->data;
		if (achptr == bchptr)
		    chptr = achptr;
	    }
	}

	if (chptr)
	    sendto_lev(DCCSEND_LEV,
		       "%s (%s@%s) sending forbidden filetyped file %s to %s (channel %s)",
		       from->name, from->user->username,
		       from->user->host, tmpfn, to->name, chptr->chname);
	else
	    sendto_lev(DCCSEND_LEV,
		       "%s (%s@%s) sending forbidden filetyped file %s to %s",
		       from->name, from->user->username, from->user->host, tmpfn, to->name);

	return 1;
    }
    return 0;
}

==> trircd-release-5.7.3-r.tar.gz: trircd-release-5.7.3-r/server/s_misc.c <==
/************************************************************************
 *   IRC - Internet Relay Chat, server/s_misc.c
 *
 *   Copyright (C) 2000-2003 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * $Id: s_misc.c,v 1.5 2004/12/25 01:19:20 tr-ircd Exp $ 
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "h.h"
#include "numeric.h"

static char *months[] = {
    "January", "February", "March", "April",
    "May", "June", "July", "August",
    "September", "October", "November", "December"
};

static char *weekdays[] = {
    "Sunday", "Monday", "Tuesday", "Wednesday",
    "Thursday", "Friday", "Saturday"
};

/*
 * stats stuff
 */
char *date(time_t aClock)
{
static char buf[80], plus;
struct tm *lt, *gm;
struct tm gmbuf;
int minswest;

    if (!aClock)
	time(&aClock);
    gm = gmtime(&aClock);
    memcpy((char *) &gmbuf, (char *) gm, sizeof(gmbuf));
    gm = &gmbuf;
    lt = localtime(&aClock);

    if (lt->tm_yday == gm->tm_yday)
	minswest = (gm->tm_hour - lt->tm_hour) * 60 + (gm->tm_min - lt->tm_min);
    else if (lt->tm_yday > gm->tm_yday)
	minswest = (gm->tm_hour - (lt->tm_hour + 24)) * 60;
    else
	minswest = ((gm->tm_hour + 24) - lt->tm_hour) * 60;

    plus = (minswest > 0) ? '-' : '+';
    if (minswest < 0)
	minswest = -minswest;

    (void) ircsprintf(buf, "%s %s %d %04d -- %02d:%02d %c%02d:%02d",
		      weekdays[lt->tm_wday], months[lt->tm_mon],
		      lt->tm_mday, lt->tm_year + 1900, lt->tm_hour,
		      lt->tm_min, plus, minswest / 60, minswest % 60);

    return buf;
}

/*
 */
char *smalldate(time_t aClock)
{
static char buf[MAX_DATE_STRING + 1];
struct tm *lt, *gm;
struct tm gmbuf;

    if (!aClock)
        time(&aClock);
    gm = gmtime(&aClock);
    memcpy((char *) &gmbuf, (char *) gm, sizeof(gmbuf));
    gm = &gmbuf;

    lt = localtime(&aClock);
    strftime(buf, MAX_DATE_STRING, "%F %T", lt);

    return buf;
}
